package de.unibielefeld.cebitec.gi.algogr.exl2;

import java.util.HashMap;
import java.util.Map;

public class HMM<State, Alphabet> {
    private Alphabet[] sigma;
    private State[] Q;
    private double[][] A;
    private double[][] E;
    private double[] alpha;

    private Map<Alphabet, Integer> indxMapAlphabet;
    private Map<State, Integer> indxMapStates;

    /**
     * Constructor of an HMM.
     * 
     * @param sigma
     *            Alphabet
     * @param Q
     *            set of states
     * @param A
     *            transition matrix
     * @param E
     *            emission probabilities
     * @param alpha
     *            transition probabilities from begin state
     */
    public HMM(Alphabet[] sigma, State[] Q, double[][] A, double[][] E,
        double[] alpha) {
        this.sigma = sigma;
        this.Q = Q;
        this.A = A;
        this.E = E;
        this.alpha = alpha;
        this.initIndex();
    }

    public Alphabet[] getAlphabet() {
        return this.sigma;
    }

    /**
     * Getter for emission probabilities.
     * 
     * @return emission probabilities
     */
    public double[][] getEmissionMatrix() {
        return this.E;
    }

    /**
     * Getter for initial transition distribution from the start state.
     * 
     * @return state state transition distribution
     */
    public double[] getBeginDistribution() {
        return this.alpha;
    }

    /**
     * Getter for set of states of the HMM.
     * 
     * @return set of states of the HMM
     */
    public State[] getStates() {
        return this.Q;
    }

    /**
     * Getter for transition matrix A
     * 
     * @return transition matrix A
     */
    public double[][] getTransitionMatrix() {
        return this.A;
    }

    /**
     * Helper function to obtain the index position of a given symbol in the
     * alphabet.
     * 
     * @param alph
     *            symbol of the alphabet
     * @return index position of alph in the alphabet list
     */
    public int indexOfSymbol(Alphabet alph) {
        return this.indxMapAlphabet.get(alph);
    }

    /**
     * Helper function to obtain the index position of a given state in the
     * state list.
     * 
     * @param state
     *            state of the HMM
     * @return index position of state in the state list
     */
    public int indexOfState(State state) {
        return this.indxMapStates.get(state);
    }

    private void initIndex() {
        this.indxMapAlphabet = new HashMap<Alphabet, Integer>();
        for (int i = 0; i < this.sigma.length; i++) {
            this.indxMapAlphabet.put(this.sigma[i], i);
        }
        this.indxMapStates = new HashMap<State, Integer>();
        for (int i = 0; i < this.Q.length; i++) {
            this.indxMapStates.put(this.Q[i], i);
        }
    }
}
